@@ -1,8 +1,10 @@
 from .test_helpers import (
     TestVectorSimulator,
+    config_model_with_cmdline_opts,
     mk_test_case_table,
     run_sim,
     run_test_vector_sim,
 )
+from .test_masters import TestMasterCL
 from .test_sinks import TestSinkCL
 from .test_srcs import TestSrcCL
